package a.b.c.d;

import java.io.Serializable;
import java.util.Date;

import javax.jdo.annotations.Index;
import javax.jdo.annotations.Unique;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Lob;
import javax.persistence.MappedSuperclass;
import javax.persistence.Persistence;
import javax.persistence.PrePersist;
import javax.persistence.PreUpdate;
import javax.persistence.SequenceGenerator;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.TypedQuery;
import javax.persistence.Version;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import com.objectdb.Enhancer;

public class SimpleObjectDBTest {

	public static void main(String[] args) throws Exception{

		enhanceClasses();
		
		EntityManagerFactory emf = Persistence
		.createEntityManagerFactory("test-company.odb");
		String companyId = "test"+System.currentTimeMillis();
		Company company = new Company();
		company.setCompanyId(companyId);
		company.setName("Test Company"+companyId);
		
		EntityManager em = emf.createEntityManager();
		em.getTransaction().begin();		
		em.persist(company);		
		System.out.println("Saved company with id: " + company.getId());
		em.getTransaction().commit();
		
		em.getTransaction().begin();
		// Fetch
		System.out.println("Loading company with companyId: " + companyId);
		TypedQuery<Company> createQuery = 
			em.createQuery("select c from Company c where c.companyId=:companyId",
				Company.class);
		createQuery.setParameter("companyId", companyId);
		Company loadedCompany = createQuery.getSingleResult();
		System.out.println("Loaded company with companyId: " + companyId +" and id: " + loadedCompany.getId());
		// Update the loaded company
		String newName = loadedCompany.getName()+" " + System.currentTimeMillis();
		System.out.println("Updating company.name to: " + newName);
		loadedCompany.setName(newName);
		em.getTransaction().commit();
		emf.close();
	}

	/**
	 * 
	 */
	private static void enhanceClasses() {
		Enhancer.enhance(BaseEntity.class.getName()+"," +Company.class.getName());
	}
	
	@MappedSuperclass
	public static abstract class BaseEntity implements Serializable{

	    private static final long serialVersionUID = -7979088681300801112L;

	    @Column (name = "opt_lock")
	    @Version()
	    private Long optimisticLock;


	    /**
	     * The date the entity was created
	     */
	    @Column(name = "created_at")
	    @Temporal(TemporalType.TIMESTAMP)
	    private Date createdDate;

	    /**
	     * The date that the entity was updated
	     */
	    @Column(name = "updated_at")
	    @Temporal(TemporalType.TIMESTAMP)
	    private Date lastUpdatedDate;    

	    public abstract Long getId();

	    public Long getOptimisticLock() {
	        return optimisticLock;
	    }

		/**
		 * @return the createdDate
		 */
		public Date getCreatedDate() {
			return createdDate;
		}

		/**
		 * @param createdDate the createdDate to set
		 */
		public void setCreatedDate(Date createdDate) {
			this.createdDate = createdDate;
		}

		/**
		 * @return the lastUpdatedDate
		 */
		public Date getLastUpdatedDate() {
			return lastUpdatedDate;
		}

		/**
		 * @param lastUpdatedDate the lastUpdatedDate to set
		 */
		public void setLastUpdatedDate(Date lastUpdatedDate) {
			this.lastUpdatedDate = lastUpdatedDate;
		}

		/**
		 * @param optimisticLock the optimisticLock to set
		 */
		public void setOptimisticLock(Long optimisticLock) {
			this.optimisticLock = optimisticLock;
		}
		
	    @PrePersist
	    public void save(){
	    	createdDate = new Date();
		}
	    
		@PreUpdate
	    public void update(){
			System.out.println(" ********* Called update() **********");
			lastUpdatedDate = new Date();
		}		
	}
	
	@Entity
	@SequenceGenerator(name="company_seq", initialValue=1,allocationSize=5)
	public static class Company extends BaseEntity{
		
	    @Id 
	    @GeneratedValue(strategy=GenerationType.IDENTITY, generator="company_seq")
	    private Long id;		

	    /**
		 * 
		 */
		private static final long serialVersionUID = -299617781352746768L;

		/**
	     * Name
	     */
	    @Column(nullable = false, length = 128)
	    @Unique(name="uq_company_name")
	    private String name;
	    
	    /**
	     * Short name to identify company
	     */
	    @Column(name="company_id")
	    @NotNull
	    @Size(min=3)
	    @Index(name="idx_company_id")
	    @Unique(name="uq_company_id")
	    private String companyId;

	    @Column(name = "detailed_desc")
	    @Lob
	    private String detailedDescription;

	    public String getName() {
	        return name;
	    }

	    public void setName(String name) {
	        this.name = name;
	    }

	    public String getDetailedDescription() {
	        return detailedDescription;
	    }

	    public void setDetailedDescription(String detailedDescription) {
	        this.detailedDescription = detailedDescription;
	    }

		/**
		 * @return the id
		 */
	    @Override
		public Long getId() {
			return id;
		}

		/**
		 * @return the companyId
		 */
		public String getCompanyId() {
			return companyId;
		}

		/**
		 * @param companyId the companyId to set
		 */
		public void setCompanyId(String companyId) {
			this.companyId = companyId;
		}
	}	
}